@@ -1,10 +1,11 @@
 package Sub::Uplevel;
 
-#use 5.006;
-
 use strict;
+use Carp qw/carp/;
 use vars qw($VERSION @ISA @EXPORT);
-$VERSION = '0.17_01';
+$VERSION = '0.19_02';
+$VERSION = eval $VERSION;
+
 
 # We must override *CORE::GLOBAL::caller if it hasn't already been 
 # overridden or else Perl won't see our local override later.
@@ -21,6 +22,14 @@ require Exporter;
 
 Sub::Uplevel - apparently run a function in a higher stack frame
 
+=begin wikidoc
+
+= VERSION
+
+This documentation describes version %%VERSION%%
+
+=end wikidoc
+
 =head1 SYNOPSIS
 
   use Sub::Uplevel;
@@ -73,6 +82,8 @@ you can do this:
         return @out;
     }
 
+C<uplevel> will issue a warning if C<$num_frames> is more than the current call
+stack depth.
 
 =cut
 
@@ -80,11 +91,18 @@ use vars qw/@Up_Frames $Caller_Proxy/;
 # @Up_Frames -- uplevel stack
 # $Caller_Proxy -- whatever caller() override was in effect before uplevel
 
+sub _apparent_stack_height {
+    my $height = 1; # start above this function 
+    while ( 1 ) {
+        last if ! defined scalar $Caller_Proxy->($height);
+        $height++;
+    }
+    return $height - 1; # subtract 1 for this function
+}
+
 sub uplevel {
     my($num_frames, $func, @args) = @_;
     
-    local @Up_Frames = ($num_frames, @Up_Frames );
-    
     # backwards compatible version of "no warnings 'redefine'"
     my $old_W = $^W;
     $^W = 0;
@@ -97,21 +115,38 @@ sub uplevel {
     # restore old warnings state
     $^W = $old_W;
 
+    carp "uplevel $num_frames is more than the caller stack"
+        if $num_frames >= _apparent_stack_height();
+
+    local @Up_Frames = ($num_frames, @Up_Frames );
+    
     return $func->(@args);
 }
 
-sub _normal_caller (;$) {
+sub _normal_caller (;$) { ## no critic Prototypes
     my $height = $_[0];
     $height++;
-    if( wantarray and !@_ ) {
-        return (CORE::caller($height))[0..2];
+    if ( CORE::caller() eq 'DB' ) {
+        # passthrough the @DB::args trick
+        package DB;
+        if( wantarray and !@_ ) {
+            return (CORE::caller($height))[0..2];
+        }
+        else {
+            return CORE::caller($height);
+        }
     }
     else {
-        return CORE::caller($height);
+        if( wantarray and !@_ ) {
+            return (CORE::caller($height))[0..2];
+        }
+        else {
+            return CORE::caller($height);
+        }
     }
 }
 
-sub _uplevel_caller (;$) {
+sub _uplevel_caller (;$) { ## no critic Prototypes
     my $height = $_[0] || 0;
 
     # shortcut if no uplevels have been called
@@ -191,7 +226,15 @@ found during the search
 
     # For returning values, we pass through the call to the proxy caller
     # function, just at a higher stack level
-    my @caller = $Caller_Proxy->($height + $adjust + 1);
+    my @caller;
+    if ( CORE::caller() eq 'DB' ) {
+        # passthrough the @DB::args trick
+        package DB;
+        @caller = $Sub::Uplevel::Caller_Proxy->($height + $adjust + 1);
+    }
+    else {
+        @caller = $Caller_Proxy->($height + $adjust + 1);
+    }
 
     if( wantarray ) {
         if( !@_ ) {
@@ -257,7 +300,7 @@ Michael G Schwern E<lt>schwern@pobox.comE<gt> (original author)
 =head1 LICENSE
 
 Original code Copyright (c) 2001 to 2007 by Michael G Schwern.
-Additional code Copyright (c) 2006 to 2007 by David A Golden.
+Additional code Copyright (c) 2006 to 2008 by David A Golden.
 
 This program is free software; you can redistribute it and/or modify it
 under the same terms as Perl itself.
@@ -271,5 +314,4 @@ Tcl's uplevel() at http://www.scriptics.com/man/tcl8.4/TclCmd/uplevel.htm
 
 =cut
 
-
 1;
